	page	80, 128

	TITLE	EPOCH.ASM

	.286c

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;	File:	epoch.asm
;	Date:	01-May-1998
;	Vers:	0.1
;	Auth:	PJD  Copyright (C) Papa Delta Limited.
;
;	Description:
;	~~~~~~~~~~~~
;	Contains the single EPOCH() function which can be called from
;	Clipper S87 to get or set the EPOCH value (which is a feature
;	not supported in this version of Clipper).
;
;	The calling syntax is similar to the SETCOLOR() function, eg.
;
;		nCurrent_setting = EPOCH (nNew_setting)
;
;	where 'nCurrent_setting' is the existing value (normally zero
;	at startup) and 'nNew_setting' [optional] is the desired EPOCH
;	setting, in the range 0 to 99.
;
;	If 'nNew_setting' is omitted, or is outside the permitted range,
;	nothing will be changed and the current setting returned.
;
;	Examples:
;	~~~~~~~~~
;	this_epoch = EPOCH()
;	* Reads the current setting.
;
;	this_epoch = EPOCH (80)
;	* Sets the EPOCH to [19]80.  Two-digit years less than 80 will
;	* be assumed to refer to 21st century dates.  The current setting
;	* is returned to 'this_epoch'.
;
;	Build instructions:
;	~~~~~~~~~~~~~~~~~~~
;	Using MASM:
;
;		masm /mx epoch;
;
;	This will create the file EPOCH.OBJ which can be included as one
;	of the files to the linker, or added to an existing library file.
;	For example, to add it to CLIPPER.LIB using the MS LIB library
;	editor/manager, use:
;
;	C:\CLIPPER> lib clipper
;	Operations: +epoch
;	List file: clipper.lst
;
;	Note that you should *ALWAYS* take a backup of CLIPPER.LIB
;	before doing this....
;
;
;	Notes and Disclaimers:
;	~~~~~~~~~~~~~~~~~~~~~~
;	This file makes use of the undocumented (to my knowledge) EPOCH
;	support already existing in the Clipper library, at least as far
;	as the file CLIPPER.LIB distributed with the Summer'87 version
;	(the file I have here is 307019 bytes, dated 21/12/87 02:00).
;
;	Disassembly of some of the CLIPPER.LIB modules appears to indicate
;	that the 16-bit word located above the public var '__last_error'
;	in the file '_asm.ASM' CSUPPORT module is indeed the EPOCH word.
;	This variable is static/private, so can only be accessed externally
;	by reference (through its address) in relation to the PUBLIC
;	'__last_error' variable.
;
;	THE "EPOCH" SOURCE HERE IS BASED ON THE ASSUMPTION THAT THIS
;	STATIC LOCATION HAS NOT BEEN MOVED OR REMOVED IN OTHER (IF ANY)
;	VERSIONS OF CLIPPER.LIB - USE THIS FUNCTION AT YOUR OWN RISK.
;
;	This code is provided strictly "as is" with no warranties etc.
;	that it will work, and absolutely no liability whatsoever is
;	accepted by either me or Papa Delta Limited for any disaster
;	or damage it might cause.  You have been warned!
;
;	That said, it seems to work just fine here...
;
;	Suggestions, possible support and any other correspondence
;	to:
;		pete@pdlmail.demon.co.uk
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;	These are defined in the Clipper EXTEND system
;	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	EXTRN	__parinfo:FAR
	EXTRN	__parni:FAR
	EXTRN	__retni:FAR
	EXTRN	__retl:FAR
	EXTRN	__ret:FAR

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

_DATA	SEGMENT PUBLIC 'DATA'
	ASSUME DS:_DATA
	EXTRN	__last_error:WORD	;declared in CSUPPORT module
_DATA	ENDS

DGROUP	GROUP	_DATA

EPOCH_PTR  equ	offset DGROUP:__last_error + 2

EPOCH_TEXT	SEGMENT PUBLIC WORD 'CODE'
		ASSUME CS:EPOCH_TEXT, DS:DGROUP, ES:NOTHING

	PUBLIC	EPOCH

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
						;
EPOCH	PROC	FAR				;
						;
	push	bp				;
	mov	bp, sp				;
	push	si				;
	mov	si, EPOCH_PTR			;address of 'epoch' var
	push	[si]				;old value for return
						;
	push	0				;check if a parameter passed
	call	__parinfo			;
	pop	bx				;rebalance stack
	or	ax, ax				;any params?
	jz	ep_ret				;no - just return
						;
	push	1				;else get 1st param
	call	__parni 			;
	pop	bx				;rebalance stack
	cmp	ax, 99				;check in range 00-99
	ja	ep_ret				;ignore if not
						;
	mov	[si], ax			;else install new value
						;
ep_ret: call	__retni 			;old value already stacked
	pop	ax				;rebalance stack
	pop	si				;
	pop	bp				;
	ret					;
						;
EPOCH	ENDP					;
						;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

EPOCH_TEXT	ENDS

	END

