	page	70, 128

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;	File:	lupdate.asm
;	Date:	28-mar-1998
;	Vers:	1.0
;	Auth:	PJD
;	Desc:	Parts Copyright (c) 1998 Papa Delta Limited
;
;	Replacement EXTEND.LIB Clipper module which corrects the Y2K
;	deficiencies in the standard Summer'87 release.  This version
;	assumes that any DBF datestamp before 1980 (century byte less
;	than 80) is actually a 21st century datestamp.
;
;	Revisions:
;	1.1	01-may-1998	PJD	Enhanced to take note of the
;					undoc'd EPOCH facility in S87
;					CLIPPER.LIB
;
;	Build instructions:
;
;		masm /mx lupdate,,;
;
;		lib extend
;		>operations: -+lupdate
;		>list file:  extend.lst
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
						;
BASEYEAR	equ   1980			;DBFs born here
						;
day		equ	word ptr  -2 [bp]	;local vars
dayb		equ	byte ptr  -2 [bp]	;
month		equ	word ptr  -4 [bp]	;
monthb		equ	byte ptr  -4 [bp]	;
year		equ	word ptr  -6 [bp]	;
						;
dbfseg		equ	word ptr  -8 [bp]	;seg address of DBF header
dbfoff		equ	word ptr -10 [bp]	;off address of DBF header
						;
dbfptr		equ    dword ptr -10 [bp]	;
						;
endstring	equ	byte ptr -12 [bp]	;null byte
day_units	equ	byte ptr -13 [bp]	;
day_tens	equ	byte ptr -14 [bp]	;
month_units	equ	byte ptr -15 [bp]	;
month_tens	equ	byte ptr -16 [bp]	;
cent_units	equ	byte ptr -17 [bp]	;
cent_tens	equ	byte ptr -18 [bp]	;
cent_hundreds	equ	byte ptr -19 [bp]	;
datestring	equ	byte ptr -20 [bp]	;
						;
LOCALS		equ	20			;stack space for locals
						;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
						;
	EXTRN	__dbfopen:FAR			;
	EXTRN	__dbfhead:FAR			;
	EXTRN	__retds:FAR			;
						;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ORIGINAL_VERSION  equ	0

_DATA	SEGMENT PUBLIC 'DATA'
	ASSUME DS:_DATA
	EXTRN	__last_error:WORD	;declared in CSUPPORT module
_DATA	ENDS

DGROUP	GROUP	_DATA

EPOCH_PTR  equ	offset DGROUP:__last_error + 2

LUPDATE_TEXT	SEGMENT PUBLIC 'CODE'
		ASSUME CS:LUPDATE_TEXT, DS:DGROUP, ES:NOTHING

	PUBLIC	LUPDATE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
						;
LUPDATE PROC	FAR				;
						;
	push	bp				;standard entry
	mov	bp, sp				;
	sub	sp, LOCALS			;reserve space for locals
	sub	ax, ax				;
	mov	year, ax			;zeroise date values
	mov	month, ax			;
	mov	day, ax 			;
	call	__dbfopen			;see if DBF is open/avbl
	or	ax, ax				;
	jz	no_DBF				;no
						;yes
	call	__dbfhead			;get header info address
	mov	dbfoff, ax			;
	mov	dbfseg, dx			;
	les	bx, dbfptr			;
	mov	al, es:01 [bx]			;year byte
	sub	ah, ah				;
	add	ax, 1900			;save it
    IF	ORIGINAL_VERSION
	cmp	ax, BASEYEAR			;adjust if required
    ELSE
	push	si				;
	mov	si, EPOCH_PTR			;address of 'epoch' var
	mov	dx, [si]			;
	add	dx, 1900			;
	pop	si				;
	cmp	ax, dx				;adjust if required
    ENDIF
	jae	no_adjust			;
	add	ax, 100 			;bias for 21st century
no_adjust:					;
	mov	year, ax			;
	mov	ax, es:02 [bx]			;month + day bytes
	mov	monthb, al			;save it
	mov	dayb, ah			;save it
no_DBF: 					;
	mov	ax, year			;convert to ASCII date string
	sub	dx, dx				;
	mov	cx, 10				;
	div	cx				;
	add	dl, '0'                         ;
	mov	cent_units, dl			;save remainder
	sub	dx, dx				;
	div	cx				;
	add	dl, '0'                         ;
	mov	cent_tens, dl			;save remainder
	sub	dx, dx				;
	div	cx				;
	add	dl, '0'                         ;save remainder
	mov	cent_hundreds, dl		;
	add	al, '0'                         ;
	mov	datestring, al			;save quotient
						;
	mov	ax, month			;now the month
	sub	dx, dx				;
	div	cx				;
	add	dl, '0'                         ;
	mov	month_units, dl 		;save remainder
	add	al, '0'                         ;
	mov	month_tens, al			;save quotient
						;
	mov	ax, day 			;finally the day
	sub	dx, dx				;
	div	cx				;
	add	dl, '0'                         ;
	mov	day_units, dl			;save remainder
	add	al, '0'                         ;
	mov	day_tens, al			;save quotient
						;
	mov	endstring, ah			;null terminate the string
	lea	ax, datestring			;
	push	ss				;
	push	ax				;
	call	__retds 			;return as a DATE value
	mov	sp, bp				;
	pop	bp				;
	ret					;
						;
LUPDATE ENDP					;
						;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

LUPDATE_TEXT	ENDS

	END
